@@ -20,8 +20,11 @@ def SocketDownload(sock):
             data += sock.recv(4096)
         frame_data = data[:msg_size]
         data = data[msg_size:]
-        xData = pickle.loads(frame_data, fix_imports=True, encoding="bytes")
-        return xData
+        try:
+            xData = pickle.loads(frame_data, fix_imports=True, encoding="bytes")
+            return xData
+        except:
+            print("EOF Error Caught.")
     except:
         raise ConnectionError("Connection Error")
 
@@ -167,9 +170,17 @@ class Socket_Server_Host:
 
         """
         self.running = False
-        for con in self.connections:
-            self.connections[con].shutdown()
+        keys = list(self.connections.keys())
+        for con in keys:
+            try:
+                self.connections[con].shutdown()
+            except:
+                pass
         self.connections = {}
+        try:
+            self.server.close()
+        except:
+            pass
 
     def CloseConnection(self, conID):
         """
@@ -292,7 +303,7 @@ class Socket_Question(object):
 
 class Socket_Connector:
 
-    def __init__(self, HOST, PORT, on_data_recv, on_question, on_connection_close, Heart_Beat=True, Heart_Beat_Wait=10):
+    def __init__(self, HOST, PORT, on_data_recv, on_question, on_connection_close, Heart_Beat=True, Heart_Beat_Wait=10, legacy=False, legacy_buffer_size=1024):
         """Host your own Socket server to allows connections to this computer.
 
         Parameters
@@ -319,6 +330,8 @@ class Socket_Connector:
         """
         self.running = True
         self.HOST = HOST
+        self.legacy = legacy
+        self.legacy_buffer_size = legacy_buffer_size
         self.PORT = PORT
         self.__ask_list__ = {}
         self.on_question = on_question
@@ -336,6 +349,9 @@ class Socket_Connector:
             time.sleep(self.heart_beat_wait)
 
     def ask(self, data, timeout=5):
+        if self.legacy:
+            print("Can't ask questions to legacy connections")
+            return
         tok = essentials.CreateToken(20, self.__ask_list__)
         self.__ask_list__[tok] = False
         self.send({"function_ask_question": tok, "data": data})
@@ -358,8 +374,12 @@ class Socket_Connector:
         if self.running == False:
             raise ConnectionResetError("No Connection")
         try:
-            SocketUpload(self.socket, data)
-        except:
+            if self.legacy:
+                self.socket.sendall(data)
+            else:
+                SocketUpload(self.socket, data)
+        except Exception as e:
+            print(e)
             self.shutdown()
 
     def shutdown(self):
@@ -388,16 +408,19 @@ class Socket_Connector:
 
         """
         while self.running:
-            try:
-                data = SocketDownload(self.socket)
-            except:
-                self.shutdown()
-                return
-            if type(data) == type({}) and 'heart_beat_function' in data:
-                pass
-            elif type(data) == type({}) and 'function_ask_response' in data:
-                self.__ask_list__[data['function_ask_response']] = data
-            elif type(data) == type({}) and 'function_ask_question' in data:
-                self.on_question(Socket_Question(data['data'], self, data['function_ask_question']))
+            if self.legacy:
+                self.on_data_recv(self.socket.recv(self.legacy_buffer_size))
             else:
-                self.on_data_recv(data)
\ No newline at end of file
+                try:
+                    data = SocketDownload(self.socket)
+                except:
+                    self.shutdown()
+                    return
+                if type(data) == type({}) and 'heart_beat_function' in data:
+                    pass
+                elif type(data) == type({}) and 'function_ask_response' in data:
+                    self.__ask_list__[data['function_ask_response']] = data
+                elif type(data) == type({}) and 'function_ask_question' in data:
+                    self.on_question(Socket_Question(data['data'], self, data['function_ask_question']))
+                else:
+                    self.on_data_recv(data)
\ No newline at end of file
